// SPDX-FileCopyrightText: © Matteo Settenvini <matteo.settenvini@montecristosoftware.eu>
// SPDX-License-Identifier: EUPL-1.2

use {
    delegate::delegate,
    std::borrow::Cow,
    testcontainers::{
        Image,
        core::{ContainerPort, WaitFor},
    },
    testcontainers_modules::minio,
};

const MINIO_IMAGE_TAG: &'static str = "RELEASE.2025-06-13T11-33-47Z";

pub struct MinIO {
    inner: minio::MinIO,
}

impl Image for MinIO {
    fn tag(&self) -> &str {
        MINIO_IMAGE_TAG.into()
    }

    fn ready_conditions(&self) -> Vec<WaitFor> {
        vec![WaitFor::message_on_stderr("API:")]
    }

    delegate! {
        to self.inner {
            fn name(&self) -> &str;
            fn expose_ports(&self) -> &[ContainerPort];
            fn env_vars(
                &self,
            ) -> impl IntoIterator<Item = (impl Into<Cow<'_, str>>, impl Into<Cow<'_, str>>)>;
            fn mounts(&self) -> impl IntoIterator<Item = &testcontainers::core::Mount>;
            fn copy_to_sources(&self) -> impl IntoIterator<Item = &testcontainers::CopyToContainer>;
            fn entrypoint(&self) -> Option<&str>;
            fn cmd(&self) -> impl IntoIterator<Item = impl Into<std::borrow::Cow<'_, str>>>;
            fn exec_after_start(
                &self,
                cs: testcontainers::core::ContainerState,
            ) -> Result<Vec<testcontainers::core::ExecCommand>, testcontainers::TestcontainersError>;
        }
    }
}

impl Default for MinIO {
    fn default() -> Self {
        Self {
            inner: Default::default(),
        }
    }
}
